@@ -33,6 +33,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <unistd.h>
 #include <pthread.h>
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xcomposite.h>
 #include <X11/extensions/Xrender.h>
@@ -76,7 +77,7 @@ Bool ignore_errors = False;
 Bool allowErrors = False;
 
 
-/* error handler - allows errors to be masked; otherwise calls X default handler */
+/* error_handler - allows errors to be masked; otherwise calls X default handler */
 
 int error_handler (Display *dpy, XErrorEvent *ev)
 {
@@ -86,9 +87,75 @@ int error_handler (Display *dpy, XErrorEvent *ev)
 }
 
 
+/* get_borders - calculate the borders added to a standard window by mutter */
+
+void get_borders (Window wd, int *lb, int *rb, int *tb, int *bb)
+{
+    Window chroot, *ch, nullwd;
+    XWindowAttributes wd_att, ch_att;
+    Atom type;
+    int ret_fmt, nch;
+    unsigned long items, left;
+    unsigned char *data;
+    long *lptr;
+
+    *lb = 0;
+    *rb = 0;
+    *tb = 0;
+    *bb = 0;
+
+    // try to find any GTK frame extents - used for override-redirect windows
+    if (XGetWindowProperty (dsp, wd, XInternAtom (dsp, "_GTK_FRAME_EXTENTS", False), 0L, 4L, False, 
+        XA_CARDINAL, &type, &ret_fmt, &items, &left, &data) == Success && items == 4)
+    {
+        lptr = (long *) data;
+        *lb = lptr[0];
+        *rb = lptr[1];
+        *tb = lptr[2];
+        *bb = lptr[3];
+        XFree (data);
+        return;
+    }
+
+    // get the attributes of the parent window
+    if (!XGetWindowAttributes (dsp, wd, &wd_att)) return;
+
+    // find if that window has children
+    if (!XQueryTree (dsp, wd, &chroot, &nullwd, &ch, &nch) || nch == 0) return;
+
+    // there is at least one child window, so get its attributes
+    if (!XGetWindowAttributes (dsp, ch[0], &ch_att))
+    {
+        XFree (ch);
+        return;
+    }
+
+    // now try to get the frame extents of the child
+    if (XGetWindowProperty (dsp, ch[0], XInternAtom (dsp, "_NET_FRAME_EXTENTS", False), 0L, 4L, False, 
+        XA_CARDINAL, &type, &ret_fmt, &items, &left, &data) != Success)
+    {
+        XFree (ch);
+        return;
+    }
+
+    // there is a child window with frame extents, so do the maths
+    if (items == 4)
+    {
+        lptr = (long *) data;
+        *lb = ch_att.x - lptr[0];
+        *rb = wd_att.width - (ch_att.x + ch_att.width) - lptr[1];
+        *tb = ch_att.y - lptr[2];
+        *bb = wd_att.height - (ch_att.y + ch_att.height) - lptr[3];
+    }
+
+    XFree (data);
+    XFree (ch);
+}
+
+
 /* get_image - construct the image to go in the loupe by copying from each window in turn */
 
-#define CONSTRAIN(lo,hi,max,offset)	 if (lo < 0) { offset += -lo; lo = 0; } if (hi > max) hi = max;
+#define CONSTRAIN(lo,hi,max,offset,sbord,fbord)	 if (lo < sbord) { offset += sbord - lo; lo = sbord; } if (hi > max - fbord) hi = max - fbord;
 
 void get_image (void)
 {
@@ -97,6 +164,7 @@ void get_image (void)
     Window root, nullwd, *children;
     int sx, sy, sw, sh, dx, dy, null;
     unsigned int nwins, wd;
+    int lb, rb, tb, bb;
 
     // get the location of the mouse pointer
     XQueryPointer (dsp, rootwin, &root, &nullwd, &posx, &posy, &null, &null, (unsigned *) &null);
@@ -131,8 +199,8 @@ void get_image (void)
         sh = sy + srch;
 
         // constrain loupe to screen, moving destination if needed
-        CONSTRAIN (sx, sw, screenw, dx);
-        CONSTRAIN (sy, sh, screenh, dy);
+        CONSTRAIN (sx, sw, screenw, dx, 0, 0);
+        CONSTRAIN (sy, sh, screenh, dy, 0, 0);
 
         // convert source to coords relative to window
         sx -= xatr.x;
@@ -141,8 +209,9 @@ void get_image (void)
         sh -= xatr.y;
 
         // constrain loupe to window, moving destination if needed
-        CONSTRAIN (sx, sw, xatr.width, dx);
-        CONSTRAIN (sy, sh, xatr.height, dy);
+        get_borders (children[wd], &lb, &rb, &tb, &bb);
+        CONSTRAIN (sx, sw, xatr.width, dx, lb, rb);
+        CONSTRAIN (sy, sh, xatr.height, dy, tb, bb);
 
         // convert loupe bounds to width and height
         sw -= sx;
@@ -299,7 +368,7 @@ void init_screen (void)
     if (posy >= screenh - dsth - 10) posy = screenh - dsth - 10;
 
     // create the window which will be used for the loupe
-    topwin = XCreateSimpleWindow (dsp, rootwin, posx, posy, dstw, dsth, 5, BlackPixel (dsp, scr), WhitePixel (dsp, scr));
+    topwin = XCreateSimpleWindow (dsp, rootwin, posx, posy, dstw, dsth, 0, BlackPixel (dsp, scr), WhitePixel (dsp, scr));
     XSelectInput (dsp, topwin, EVENT_MASK);
 
     if (!XCompositeQueryExtension (dsp, &event_base, &error_base))
@@ -343,7 +412,7 @@ void args (int argc, char **argv)
             {
                 if (strcmp ("--help", argv[i]) == 0)
                 {
-                    puts (  "Usage: mag-pi [OPTIONS]\n"
+                    puts (  "Usage: mage [OPTIONS]\n"
                             "Opens a screen magnifier under the mouse pointer.\n\n"
                             "\t-c [DIAMETER]\t\tSet a circular shape for the loupe\n"
                             "\t-r [WIDTH] [HEIGHT]\tSet a rectamgular shape for the loupe\n"
@@ -498,10 +567,10 @@ int main (int argc, char *argv[])
                     drag = 0;
                     system ("lxpanelctl command magnifier pos");
                 }
-                if (ev.type == MotionNotify && drag) XMoveWindow (dsp, topwin, posx - (dstw / 2) - 5, posy - (dsth / 2) - 5);
+                if (ev.type == MotionNotify && drag) XMoveWindow (dsp, topwin, posx - (dstw / 2), posy - (dsth / 2));
             }
         }
-        else XMoveWindow (dsp, topwin, posx - (dstw / 2) - 5, posy - (dsth / 2) - 5);
+        else XMoveWindow (dsp, topwin, posx - (dstw / 2), posy - (dsth / 2));
     }
 
     XCloseDisplay (dsp);
